//
//  WTTrackingMapRecorder.h
//  WikitudeNativeSDK
//
//  Created by Andreas Schacherbauer on 01/10/15.
//  Copyright © 2015 Wikitude. All rights reserved.
//

#import <Foundation/Foundation.h>


NS_ASSUME_NONNULL_BEGIN

typedef void(^WTTrackingMapRecordingStartupHandler)(BOOL isRecording, NSError * _Nullable error);
typedef void(^WTTrackingMapRecordingCompletionHandler)(NSURL * _Nullable trackingMapURL, NSError * _Nullable error);

/**
 * WTTrackingMapRecordingQuality defines the possible states in which a running 3d tracking map recording session can be.
 */
typedef NS_ENUM(NSInteger, WTTrackingMapRecordingQuality) {
    /**
     * WTTrackingMapRecordingQuality_Bad indicates that the current 3d tracking map can not be used to successfully recognize any objects in the recorded scene.
     */
    WTTrackingMapRecordingQuality_Bad = -1,

    /**
     * WTTrackingMapRecordingQuality_Average indicates that the current 3d tracking map can be used to recognize objects, but the results might not be very satisfying.
     */
    WTTrackingMapRecordingQuality_Average,

    /**
     * WTTrackingMapRecordingQuality_Good indicates that the currently recorded 3d tracking map includes enough information so that it's content can be recognized at a later point in time.
     */
    WTTrackingMapRecordingQuality_Good
};


@class WTTrackingMapRecorder;

/**
 * @Brief WTTrackingMapRecorderDelegate provides methods that inform about an active 3d tracking map recording session.
 */
@protocol WTTrackingMapRecorderDelegate <NSObject>

@optional
/**
 * @Brief Called whenever an active 3d tracking map recording session changes it's quality.
 *
 * @discussion The new recording quality might be worse than the previous one, meaning that a new value doesn't has to be better than a previous one.
 */
- (void)trackingMapRecorder:(WTTrackingMapRecorder *)trackingMapRecorder didChangeTrackingMapRecordingQualityFrom:(WTTrackingMapRecordingQuality)oldTrackingQuality toQuality:(WTTrackingMapRecordingQuality)newTrackingQuality;

@end


/**
 * WTTrackingMapRecorder provides the functionality to record so called 'Tracking Maps' that are needed for 3D tracking.
 *
 */
@interface WTTrackingMapRecorder : NSObject

/**
 * Delegate pointer for a WTTrackingMapRecorder object.
 */
@property (nonatomic, weak) id<WTTrackingMapRecorderDelegate>               delegate;

/**
 * Starts a new 3D tracking map recording session.
 *
 * @param error A pointer to a NSError object which will contain specific information about why the tracking map recording couldn't be started.
 *
 * @return YES if a new tracking map recording session could be initiated, NO otherwise.
 */
- (void)startTrackingMapRecording:(WTTrackingMapRecordingStartupHandler)startupHandler;

/**
 * Use this method to check if a 3D tracking map recording session is currently running or not.
 *
 * @return YES is a 3D tracking map recording session is currently running or not.
 */
- (BOOL)isRecording;

/**
 * Stops an already running 3D tracking map recording session.
 *
 * @param mapName The name of the 3D tracking map file.
 * @param completionHandler A completion handler that is called once the 3D tracking map could be saved to disk or not.
 */
- (void)stopTrackingMapRecording:(NSString *)mapName completion:(WTTrackingMapRecordingCompletionHandler)completionHandler;


/**
 * Cancels an already running 3D tracking map recording session. 
 * Calling this method without a running 3D tracking map recording session has no effect.
 *
 * Note: This method return immediately.
 *
 * @Discussion Once the running 3D tracking map recording session is canceled, it can't be restarted nor saved.
 */
- (void)cancelTrackingMapRecording;

@end

NS_ASSUME_NONNULL_END
